@@ -2,6 +2,7 @@ import os
 import time
 import json
 import tqdm
+import requests
 import serial
 import joblib
 import numpy as np
@@ -19,58 +20,84 @@ def _process_packet(raw_packet, sensor_data_records, sample_index, add_timestamp
     sensor_record.update({f"data_value_{i + 1}": value for i, value in enumerate(sensor_values)})
     sensor_data_records.append(sensor_record)
 
+
 def ensure_directory_exists(dir_path):
     os.makedirs(dir_path, exist_ok=True)
 
+
 class DataFetcher:
-    def __init__(self, serial_port, baud_rate=9600):
-        self.serial_connection = serial.Serial(port=serial_port, baudrate=baud_rate)
+    def __init__(self, source="serial", serial_port=None, baud_rate=9600, api_url=None):
+        """
+        Parameters:
+            source: "serial" or "http"
+            serial_port: Serial port (if using serial)
+            baud_rate: Baud rate (if using serial)
+            api_url: API endpoint (if using http)
+        """
+        self.source = source
+        self.api_url = api_url
+        self.serial_connection = None
+
+        if self.source == "serial":
+            if not serial_port:
+                raise ValueError("serial_port must be provided when using source='serial'")
+            self.serial_connection = serial.Serial(port=serial_port, baudrate=baud_rate)
 
     def close_connection(self):
         if self.serial_connection and self.serial_connection.is_open:
             self.serial_connection.close()
             print("Serial connection closed.")
 
+    def _read_packet(self):
+        """Read one packet depending on source"""
+        if self.source == "serial":
+            return json.loads(self.serial_connection.readline().decode())
+        elif self.source == "http":
+            r = requests.get(self.api_url, timeout=5)
+            return r.json()
+        else:
+            raise ValueError("Unsupported source. Use 'serial' or 'http'.")
+
     def fetch_data(self, return_as_numpy=False):
         try:
-            raw_packet = json.loads(self.serial_connection.readline().decode())
-            if return_as_numpy:
-                return np.array(raw_packet['sensorValues'])
-            else:
-                return raw_packet['sensorValues']
-        except (json.JSONDecodeError, KeyError) as e:
+            raw_packet = self._read_packet()
+            values = raw_packet.get("sensorValues", [])
+            return np.array(values) if return_as_numpy else values
+        except (json.JSONDecodeError, KeyError, requests.RequestException) as e:
             print(f"Error fetching data: {e}")
-            return [] if not return_as_numpy else np.array([])
+            return np.array([]) if return_as_numpy else []
 
     def log_sensor_data(self, class_label=None, num_samples=5, add_timestamp=False, add_count=False, output_dir="."):
-        initial_packet = json.loads(self.serial_connection.readline().decode())
+        initial_packet = self._read_packet()
         sensor_name = initial_packet.get("sensorName", "Unknown")
         file_name = f"{sensor_name}_data_log.csv"
+
+        # Choose dataset folder
         if class_label:
             directory_path = os.path.join("Dataset", str(class_label))
         else:
             directory_path = "Dataset"
 
-        # Ensure the directory exists
         ensure_directory_exists(directory_path)
 
-        # Construct the full file path
         output_file_name = os.path.join(directory_path, file_name)
 
         sensor_data_records = []
-        _process_packet(initial_packet, sensor_data_records, sample_index=0, add_timestamp=add_timestamp, add_count=add_count)
+        _process_packet(initial_packet, sensor_data_records, sample_index=0,
+                        add_timestamp=add_timestamp, add_count=add_count)
 
         print(f"Sampling {sensor_name} sensor.")
 
-        # Use tqdm for the progress bar
         for sample_index in tqdm.tqdm(range(1, num_samples), desc="Gathering data", unit="samples"):
-            raw_packet = json.loads(self.serial_connection.readline().decode())
-            _process_packet(raw_packet, sensor_data_records, sample_index, add_timestamp=add_timestamp, add_count=add_count)
+            raw_packet = self._read_packet()
+            _process_packet(raw_packet, sensor_data_records, sample_index,
+                            add_timestamp=add_timestamp, add_count=add_count)
 
         data_frame = pd.DataFrame(sensor_data_records)
         data_frame.to_csv(output_file_name, index=False)
         print(f"\nData saved to {output_file_name}")
 
+
 def load_scaler(scaler_file):
     scaler = joblib.load(scaler_file)
     return scaler.mean_.tolist(), scaler.scale_.tolist()
@@ -99,10 +126,29 @@ def _format_scaler_with_new_lines(data, values_per_line=5):
     # Join the chunks with newline
     return ',\n    '.join(lines)
 
+
 def _append_to_model_h(model_header_file, scaler_mean, scaler_scale, tflite_array, tflite_length):
     model_h_file_name = os.path.splitext(os.path.basename(model_header_file))[0]
-    scaler_mean_str = _format_scaler_with_new_lines(scaler_mean, values_per_line=5)
-    scaler_scale_str = _format_scaler_with_new_lines(scaler_scale, values_per_line=5)
+
+    # Prepare the scaler data if available
+    scaler_section = ""
+    if scaler_mean is not None and scaler_scale is not None:
+        scaler_mean_str = _format_scaler_with_new_lines(scaler_mean, values_per_line=5)
+        scaler_scale_str = _format_scaler_with_new_lines(scaler_scale, values_per_line=5)
+
+        scaler_section = f"""
+// Scaler Data
+// -----------------------------
+// Mean and scale values extracted from scaler.pkl
+// These are used for preprocessing input data before inference.
+const float scaler_mean[] = {{
+    {scaler_mean_str}
+}};
+
+const float scaler_scale[] = {{
+    {scaler_scale_str}
+}};
+"""
 
     # Prepare the content for model.h
     model_h_content = f"""/*
@@ -112,7 +158,7 @@ def _append_to_model_h(model_header_file, scaler_mean, scaler_scale, tflite_arra
  * a platform specializing in TinyML for edge devices.
  *
  * It contains both the TensorFlow Lite (TFLite) model data
- * and the scaler details for preprocessing.
+ * and the scaler details for preprocessing (if available).
  *
  * Usage: Deploy this header file on your microcontroller or
  * edge device to enable real-time inference with the EdgeNeuron framework.
@@ -131,17 +177,8 @@ unsigned char {model_h_file_name}[] = {{
 }};
 unsigned int {model_h_file_name}_len = {tflite_length};
 
-// Scaler Data
-// -----------------------------
-// Mean and scale values extracted from scaler.pkl
-// These are used for preprocessing input data before inference.
-const float scaler_mean[] = {{
-    {scaler_mean_str}
-}};
+{scaler_section}
 
-const float scaler_scale[] = {{
-    {scaler_scale_str}
-}};
 #endif  // EDGE_NEURON_MODEL_H
 """
 
